// Copyright (c) 2015-2016, XMOS Ltd, All rights reserved
// Teig: Taken from AN00220

// Used by lib_mic_array in:
//                 decimate_to_pcm_4ch.S asm
//                     mic_array_frame.h
//                              pdm_rx.S asm
// Somehow it's also seen by
//    AN00220_app_phase_aligned_example.xc

#ifndef MIC_ARRAY_CONF_H_
    #define MIC_ARRAY_CONF_H_

    #define MIC_ARRAY_MAX_FRAME_SIZE_LOG2 3  // 0 : 2 exp(0) = 1 sample  per microphone in audio.data in example
                                             // 1 : 2 exp(1) = 2 samples per microphone in audio.data in example
                                             // 2 : 2 exp(2) = 4 samples per microphone in audio.data in example
                                             // 3 : 2 exp(3) = 8 samples per microphone in audio.data in example

    #define MIC_ARRAY_NUM_SAMPLES (1 << MIC_ARRAY_MAX_FRAME_SIZE_LOG2)

    // The four channel decimators (mic_array_pdm_rx()) output frames of either
    // time domain audio or FFT ready audio prepared for an FFT. The define
    // MIC_ARRAY_MAX_FRAME_SIZE_LOG2 (found in mic_array_conf.h) should be used
    // to allocate the arrays to store the frames. This means that all frames
    // structures will allocate enough memory to allow for a frame size of
    // two to the power of MIC_ARRAY_MAX_FRAME_SIZE_LOG2 regardless of the size
    // used in the decimator_config_common.
    // It is recommended that the frame_size_log2 field of decimator_config_common
    // is always set to MIC_ARRAY_MAX_FRAME_SIZE_LOG2. Equally, the define MIC_ARRAY_NUM_MICS
    // is used for allocating the memory for the frame structure. This must be set to a multiple of 4.

    #define MIC_ARRAY_NUM_MICS 4 // or 8
                                 // But the mic_array board has 7 Infineon IM69D130 MEMS microphones
                                 // "Count of microphones(channels) must be set to a multiple of 4"
#else
    #error NESTED .h FILE
#endif /* MIC_ARRAY_CONF_H_ */
